package gui;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.filechooser.FileSystemView;

import util.Runner;

public class EventHandler implements ActionListener {

	public enum NexusMode {
		CAPTURE, PLAY
	}

	public enum PlaybackMode {
		NOTHING, SIGNAL_ONLY, SIGNAL_AND_MEDIA
	}

	protected NexusMode nexusMode;
	protected PlaybackMode playbackMode;

	protected FrontEnd localpanel;

	public EventHandler(FrontEnd panel) {
		this.localpanel = panel;

		nexusMode = NexusMode.PLAY;
		playbackMode = PlaybackMode.SIGNAL_AND_MEDIA;

		switch (localpanel.getGuiMode()) {
		case EVALUATION_MODE:
			break;
		case DIST_MODE:
		case EXPERIMENT_MODE:
			nexusMode = NexusMode.CAPTURE;
			break;
		}
	}

	public void actionPerformed(ActionEvent event) {
		switch (localpanel.getGuiMode()) {
		case EVALUATION_MODE:
			// ////////////
			// PLAY BUTTON
			// ////////////
			if (event.getActionCommand().equals(FrontEnd.FRONTEND_PLAY_BUTTON)) {
				nexusMode = NexusMode.PLAY;
				localpanel.jComboBox_signal.setEditable(false);
			}

			// ///////////////
			// CAPTURE BUTTON
			// ///////////////
			if (event.getActionCommand().equals(
					FrontEnd.FRONTEND_CAPTURE_BUTTON)) {
				nexusMode = NexusMode.CAPTURE;
				localpanel.jComboBox_signal.setEditable(true);
			}
		case EXPERIMENT_MODE:
			// ///////////
			// OFF BUTTON
			// ///////////
			if (event.getActionCommand().equals(FrontEnd.FRONTEND_OFF_BUTTON)) {
				playbackMode = PlaybackMode.NOTHING;
				localpanel.trainingModeIcon.setIcon(new ImageIcon("no_output.jpg"));
			}

			// ///////////////
			// SIGNAL BUTTON
			// ///////////////
			if (event.getActionCommand()
					.equals(FrontEnd.FRONTEND_SIGNAL_BUTTON)) {
				playbackMode = PlaybackMode.SIGNAL_ONLY;
				localpanel.trainingModeIcon.setIcon(new ImageIcon("signals.jpg"));
			}

			// /////////////
			// MEDIA BUTTON
			// /////////////
			if (event.getActionCommand().equals(FrontEnd.FRONTEND_MEDIA_BUTTON)) {
				playbackMode = PlaybackMode.SIGNAL_AND_MEDIA;
				localpanel.trainingModeIcon.setIcon(new ImageIcon("media.jpg"));
			}
		case DIST_MODE:
			if (event.getActionCommand().equals(FrontEnd.FRONTEND_MEDIA_USE_HISTORY)) {
				localpanel.mediaFileTextField.setText(localpanel.jSpinner_history.getValue().toString());
			}
		}

		// ////////////
		// MAIN BUTTON
		// ////////////
		if (event.getActionCommand().equals(FrontEnd.FRONTEND_MAIN_BUTTON)) {
			if (localpanel.toggle_start.getText() == "Start") {
				localpanel.toggle_start.setText("Stop");
				localpanel.toggle_start.setBackground(Color.RED);
				localpanel.saveProperties();
				Runner.startExercise(nexusMode, playbackMode);
			} else {
				Runner.killStartedApps();
			}
		}

		// //////////
		// FILE OPEN
		// //////////
		if (event.getActionCommand().equals(FrontEnd.FRONTEND_CHOOSE_FILE)) {
			JFileChooser fc = ((FrontEnd) localpanel).mediaFileChooser;
			int returnval = fc.showOpenDialog(localpanel);
			if (returnval == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				if (file.canRead()) {
					((FrontEnd) localpanel).mediaFileTextField.setText(file
							.getAbsolutePath());
				} else {
					JOptionPane.showMessageDialog(localpanel,
							"Chosen file not readable!", "Warning",
							JOptionPane.WARNING_MESSAGE);
				}
			}
		}

		// ///////////////////////
		// FILE TEXTFIELD CHANGED
		// ///////////////////////
		if (event.getActionCommand().equals(FrontEnd.FRONTEND_FILETEXT_CHANGE)) {
			JTextField tf = ((FrontEnd) localpanel).mediaFileTextField;
			String filename = tf.getText();
			File file = new File(filename);
			if (!file.canRead()) {
				file = new File(System.getProperty("user.dir") + File.separator
						+ filename);
				if (file.canRead()) {
					tf.setText(file.getAbsolutePath());
				} else {
					file = new File(System.getProperty("user.dir")
							+ File.separator + "media" + File.separator
							+ filename);
					if (file.canRead()) {
						tf.setText(file.getAbsolutePath());
					} else {
						tf.setText("");
						JOptionPane.showMessageDialog(localpanel,
								"Chosen file not readable!", "Warning",
								JOptionPane.WARNING_MESSAGE);
					}
				}
			}
		}

		// ///////////////////
		// CHOOSER MODE: FILE
		// ///////////////////
		if (event.getActionCommand().equals(
				FrontEnd.FRONTEND_MEDIACHOOSER_FILEMODE)) {
			((FrontEnd) localpanel).play_file = true;
			((FrontEnd) localpanel).mediaChooserPanel.setVisible(true);
			((FrontEnd) localpanel).mediaFileTextField.setText("");
			localpanel.validate();
		}

		// //////////////////
		// CHOOSER MODE: DVD
		// //////////////////
		if (event.getActionCommand().equals(
				FrontEnd.FRONTEND_MEDIACHOOSER_DVDMODE)) {
			((FrontEnd) localpanel).play_file = false;
			((FrontEnd) localpanel).mediaChooserPanel.setVisible(false);
			localpanel.validate();

			File[] roots = File.listRoots();
			FileSystemView view = FileSystemView.getFileSystemView();

			boolean foundDriveWithDisk = false;
			for (File f : roots) {
				if ("CD-Laufwerk".equals(view.getSystemTypeDescription(f))) {
					if (f.canRead()) {
						foundDriveWithDisk = true;
						((FrontEnd) localpanel).mediaFileTextField.setText(f
								.getAbsolutePath());
						break;
					}
				}
			}
			if (!foundDriveWithDisk) {
				JOptionPane.showMessageDialog(localpanel,
						"Could not find drive with disk!", "Warning",
						JOptionPane.WARNING_MESSAGE);
			}
		}

	}
}
