package gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerListModel;
import javax.swing.border.TitledBorder;

import util.Runner;


@SuppressWarnings("serial")
public class FrontEnd extends JPanel {

	public static final String FRONTEND_MAIN_BUTTON = "MainButtonPressed";
	public static final String FRONTEND_OFF_BUTTON = "OffButtonPressed";
	public static final String FRONTEND_SIGNAL_BUTTON = "SignalButtonPressed";
	public static final String FRONTEND_MEDIA_BUTTON = "MediaButtonPressed";
	public static final String FRONTEND_PLAY_BUTTON = "PlayButtonPressed";
	public static final String FRONTEND_CAPTURE_BUTTON = "CaptureButtonPressed";
	public static final String FRONTEND_CHOOSE_FILE = "FileChooserRequested";
	public static final String FRONTEND_FILETEXT_CHANGE = "FileTextFieldChanged";
	public static final String FRONTEND_MEDIACHOOSER_FILEMODE = "ChooseMediaFile";
	public static final String FRONTEND_MEDIACHOOSER_DVDMODE = "ChooseMediaDVD";
	public static final String FRONTEND_MEDIA_USE_HISTORY = "UseHistoryURL";

	public JButton toggle_start = null;
	public JRadioButton button_play = null;
	public JRadioButton button_capture = null;
	public JRadioButton button_off = null;
	public JRadioButton button_black = null;
	public JRadioButton button_visual = null;
	public String mediaurl = null;
	public String signalurl = null;
	public JSpinner jSpinner_time = null;
	public JSpinner jSpinner_age = null;
	public JSpinner jSpinner_history = null;
	public JLabel jLabel_signals;
	public JComboBox jComboBox_media = null;
	public JComboBox jComboBox_signal = null;
	public JLabel trainingModeIcon = null;
	
	protected EventHandler eventhandler = null;
	public JPanel mediaChooserPanel;
	public JFileChooser mediaFileChooser;
	public JTextField mediaFileTextField;
	public boolean play_file = true;
	protected String media_file_basename;
	protected JPanel signalPanel;
	
	protected static final String PROPDIR = System.getProperty("user.home") + System.getProperty("file.separator") + "LMEErgoFeeder" + System.getProperty("file.separator");
	protected static final String PROPFILE = PROPDIR + "user.properties";
	protected Properties userProps;
	
	public enum Mode {
		EVALUATION_MODE,
		EXPERIMENT_MODE,
		DIST_MODE
	}
	
	protected Mode guiMode;
	
	public Mode getGuiMode() {
		return guiMode;
	}

	public FrontEnd() {
		this(Mode.EVALUATION_MODE);
	}
	
	public FrontEnd(FrontEnd.Mode mode) {
		guiMode = mode;
		initialize();
		this.setVisible(true);
		Runner.localpanel = this;
	}
	protected void initialize() {
		userProps = new Properties();
		switch (guiMode) {
		case DIST_MODE:
			File dir = new File(PROPDIR);
			if (!dir.exists()) {
				if (!dir.mkdir()) {
					JOptionPane.showMessageDialog(this,
							"Cannot create " + PROPDIR + " and it does not exist! Exiting!", "Warning",
							JOptionPane.WARNING_MESSAGE);
					System.exit(1);
				}
			}
			File propfile = new File(PROPFILE);
			if(!propfile.exists()) {
				try {
					if (!propfile.createNewFile()) {
						JOptionPane.showMessageDialog(this,
								"Cannot create " + PROPFILE + " and it does not exist! Exiting!", "Warning",
								JOptionPane.WARNING_MESSAGE);
						System.exit(1);
					}
				} catch (HeadlessException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			try {
				userProps.load(new FileReader(PROPFILE));
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		addEventHandler();
		initGUI();
	}
	
	protected void initGUI() {
		switch (guiMode) {
		case EVALUATION_MODE:
			this.setLayout(new GridLayout(1, 2));
			break;
		case DIST_MODE:
		case EXPERIMENT_MODE:
			this.setFont(new Font("Tahoma", Font.PLAIN, 24));
			this.setLayout(new GridLayout(2,1));
			break;
		}

		this.add(initExercisePanel());

		JPanel layoutHelperPanel = new JPanel();
		layoutHelperPanel.setLayout(new GridLayout(2, 1));
		
		switch (guiMode) {
		case EVALUATION_MODE:
			layoutHelperPanel.add(initNexusPanel());
		}
		
		setComponentFonts(this);
		
		layoutHelperPanel.add(initStartButton());
		this.add(layoutHelperPanel);

		// file chooser stuff
		mediaFileChooser = new JFileChooser();
		mediaFileChooser.setCurrentDirectory(new File("./media"));
	}

	private JPanel initExercisePanel() {
		// ////////////////
		// Exercise Stuff
		// ////////////////
		JPanel exercisePanel = new JPanel();
		switch (guiMode) {
		case DIST_MODE:
		case EVALUATION_MODE:
			exercisePanel.setLayout(new GridLayout(0, 1));
			break;
		case EXPERIMENT_MODE:
			exercisePanel.setLayout(new GridLayout(0, 2));
			break;
		}
		
		TitledBorder exerciseBorder = BorderFactory
				.createTitledBorder(
						BorderFactory.createLineBorder(Color.BLACK),
						"Training options");
		exerciseBorder.setTitleJustification(TitledBorder.CENTER);
		exercisePanel.setBorder(exerciseBorder);
		// Media File Selection
		{
			JPanel mediaFilePanel = new JPanel();
			mediaFilePanel.setLayout(new GridLayout(0,1));
			mediaFilePanel.setBorder(BorderFactory.createTitledBorder(
					BorderFactory.createLineBorder(Color.BLACK),
					"Media Selection"));

			// JLabel mediaFileLabel = new JLabel("Choose media:");
			// mediaFilePanel.add(mediaFileLabel);

			JPanel mediaChooserModePanel = new JPanel();

			JRadioButton chooserMode_file = new JRadioButton();
			chooserMode_file.setText("Media File");
			chooserMode_file.setActionCommand(FRONTEND_MEDIACHOOSER_FILEMODE);
			chooserMode_file.addActionListener(eventhandler);
			chooserMode_file.setSelected(true);
			mediaChooserModePanel.add(chooserMode_file);

			JRadioButton chooserMode_dvd = new JRadioButton();
			chooserMode_dvd.setText("DVD");
			chooserMode_dvd.setActionCommand(FRONTEND_MEDIACHOOSER_DVDMODE);
			chooserMode_dvd.addActionListener(eventhandler);
			mediaChooserModePanel.add(chooserMode_dvd);

			ButtonGroup mediaChooserGroup = new ButtonGroup();
			mediaChooserGroup.add(chooserMode_file);
			mediaChooserGroup.add(chooserMode_dvd);

			mediaFilePanel.add(mediaChooserModePanel);

			mediaChooserPanel = new JPanel();
			mediaFileTextField = new JTextField(20);
			mediaChooserPanel.add(mediaFileTextField);
			mediaFileTextField.setActionCommand(FRONTEND_FILETEXT_CHANGE);
			mediaFileTextField.addActionListener(eventhandler);

			JButton mediaOpenFileButton = new JButton("Open file...");
			mediaOpenFileButton.setActionCommand(FRONTEND_CHOOSE_FILE);
			mediaOpenFileButton.addActionListener(eventhandler);
			mediaChooserPanel.add(mediaOpenFileButton);
			
			mediaFilePanel.add(mediaChooserPanel);
			
			if (guiMode == Mode.DIST_MODE) {
				JPanel historyPanel = new JPanel();
				String[] historyStrings = new String[5];
				for (int i = 1; i <= 5; i++) {
					historyStrings[i - 1] = userProps.getProperty("lastMedia" + i, "");
				}
				SpinnerListModel historyModel = new SpinnerListModel(historyStrings);
				jSpinner_history = new JSpinner();
				jSpinner_history.setModel(historyModel);
				historyPanel.add(jSpinner_history);
				JButton button = new JButton("Use as media");
				button.setActionCommand(FRONTEND_MEDIA_USE_HISTORY);
				button.addActionListener(eventhandler);
				historyPanel.add(button);
				mediaFilePanel.add(historyPanel);
			}

			exercisePanel.add(mediaFilePanel);
		}

		// age selection
		{
			JPanel agePanel = new JPanel();
			agePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
					.createLineBorder(Color.BLACK), "Select Age"));

			String[] agearray = new String[70];
			for (int i = 0; i < agearray.length; i++) {
				agearray[i] = Integer.toString(i + 18);
			}

			SpinnerListModel jSpinner_ageModel = new SpinnerListModel(agearray);
			jSpinner_age = new JSpinner();
			agePanel.add(jSpinner_age);
			jSpinner_age.setModel(jSpinner_ageModel);
			jSpinner_age.setValue(userProps.getProperty("age", "18"));

			JLabel ageUnitLabel = new JLabel("years");
			agePanel.add(ageUnitLabel);

			exercisePanel.add(agePanel);
		}

		// training time
		{
			JPanel trainingTimePanel = new JPanel();
			trainingTimePanel.setBorder(BorderFactory.createTitledBorder(
					BorderFactory.createLineBorder(Color.BLACK),
					"Set Training Time"));

			String[] timearray = new String[20];
			for (int i = 0; i < timearray.length; i++) {
				timearray[i] = Integer.toString(i + 1);
			}
			SpinnerListModel jSpinner_timeModel = new SpinnerListModel(
					timearray);

			jSpinner_time = new JSpinner();
			trainingTimePanel.add(jSpinner_time);
			jSpinner_time.setModel(jSpinner_timeModel);
			jSpinner_time.setValue(userProps.getProperty("trainingtime", timearray[timearray.length - 1]));

			JLabel timeUnitLabel = new JLabel("minute(s)");
			trainingTimePanel.add(timeUnitLabel);

			exercisePanel.add(trainingTimePanel);
		}

		// training mode
		switch (guiMode) {
		case DIST_MODE:
			break;
		case EXPERIMENT_MODE:
		case EVALUATION_MODE:
		{
			JPanel trainingModePanel = new JPanel();
			trainingModePanel.setBorder(BorderFactory.createTitledBorder(
					BorderFactory.createLineBorder(Color.BLACK),
					"Training Mode"));

			trainingModePanel.setLayout(new GridLayout(1, 2));
			trainingModeIcon = new JLabel(new ImageIcon("media.jpg"));
			
			JPanel trainingModeButtons = new JPanel();
			trainingModeButtons.setLayout(new GridLayout(3,1));

			trainingModePanel.add(trainingModeButtons);
			trainingModePanel.add(trainingModeIcon);
			
			button_off = new JRadioButton();
			button_off.setText("No graphical output");
			button_off.setActionCommand(FRONTEND_OFF_BUTTON);
			button_off.addActionListener(eventhandler);
			trainingModeButtons.add(button_off);

			button_black = new JRadioButton();
			button_black.setText("Biosignals only");
			button_black.setActionCommand(FRONTEND_SIGNAL_BUTTON);
			button_black.addActionListener(eventhandler);
			trainingModeButtons.add(button_black);

			button_visual = new JRadioButton();
			button_visual.setText("Media and biosignals");
			button_visual.setSelected(true);
			button_visual.setActionCommand(FRONTEND_MEDIA_BUTTON);
			button_visual.addActionListener(eventhandler);
			trainingModeButtons.add(button_visual);

			ButtonGroup trainingModeGroup = new ButtonGroup();
			trainingModeGroup.add(button_off);
			trainingModeGroup.add(button_black);
			trainingModeGroup.add(button_visual);

			exercisePanel.add(trainingModePanel);
		}
		}
		return exercisePanel;
	}

	protected JPanel initNexusPanel() {
		// /////////////
		// NEXUS-STUFF
		// ////////////
		JPanel nexusPanel = new JPanel();
		nexusPanel.setLayout(new GridLayout(0, 1));
		TitledBorder nexusBorder = BorderFactory.createTitledBorder(
				BorderFactory.createLineBorder(Color.BLACK),
				"Biosignal options");
		nexusBorder.setTitleJustification(TitledBorder.CENTER);
		nexusPanel.setBorder(nexusBorder);
		// mode selection
		{
			JPanel modePanel = new JPanel(new GridLayout(2, 1));
			modePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
					.createLineBorder(Color.BLACK), "Mode"));

			// Play-radiobutton
			button_play = new JRadioButton();
			button_play.setText("Play");
			button_play.setSelected(true);
			button_play.setActionCommand(FRONTEND_PLAY_BUTTON);
			button_play.addActionListener(eventhandler);
			modePanel.add(button_play);

			// capture-radiobutton
			button_capture = new JRadioButton();
			button_capture.setText("Capture");
			button_capture.setActionCommand(FRONTEND_CAPTURE_BUTTON);
			button_capture.addActionListener(eventhandler);
			modePanel.add(button_capture);

			ButtonGroup signalmodegroup = new ButtonGroup();
			signalmodegroup.add(button_play);
			signalmodegroup.add(button_capture);

			nexusPanel.add(modePanel);
		}

		// Signal file selection
		{
			signalPanel = new JPanel(new GridLayout(2, 1));
			signalPanel.setBorder(BorderFactory.createTitledBorder(
					BorderFactory.createLineBorder(Color.BLACK), "Signalfile"));

			jLabel_signals = new JLabel("Select File to play");
			signalPanel.add(jLabel_signals);

			signalurl = "./recordings/";
			File signaldir = new File(signalurl);
			String[] signalfiles = signaldir.list(new FilenameFilter() {

				@Override
				public boolean accept(File dir, String name) {
					File tmp = new File(dir.getAbsolutePath() + System.getProperty("file.separator") + name);
					return !tmp.isDirectory() && name.endsWith(".nexus");
				}
				
			});

			ComboBoxModel jComboBox_signalModel = new DefaultComboBoxModel(
					signalfiles);
			jComboBox_signal = new JComboBox();
			signalPanel.add(jComboBox_signal);
			jComboBox_signal.setModel(jComboBox_signalModel);
			jComboBox_signal.setEditable(false);

			nexusPanel.add(signalPanel);
		}
		return nexusPanel;
	}

	protected JPanel initStartButton() {
		// ///////////////////
		// Start Button Foo
		// ///////////////////
		JPanel startButtonPanel = new JPanel();
		startButtonPanel.setLayout(new BoxLayout(startButtonPanel,
				BoxLayout.PAGE_AXIS));
		// start button foo
		{
			toggle_start = new JButton();
			toggle_start.setText("Start");
			toggle_start.setBackground(Color.GREEN);
			toggle_start.setFont(new Font("Tahoma", Font.BOLD, 20));
			toggle_start.setAlignmentX(CENTER_ALIGNMENT);
			toggle_start.setActionCommand(FRONTEND_MAIN_BUTTON);
			toggle_start.addActionListener(eventhandler);

			startButtonPanel.add(Box.createVerticalGlue());
			startButtonPanel.add(toggle_start);
			startButtonPanel.add(Box.createVerticalGlue());
		}
		return startButtonPanel;
	}
	
	private void splitMediaFilename() {
		File file = new File(mediaFileTextField.getText());
		if (play_file == false) {
			mediaurl = "";
			media_file_basename = file.getAbsolutePath();
		} else {
			if (!file.canRead()) {
				try {
					URL url = new URL(mediaFileTextField.getText());
					mediaurl = "";
					media_file_basename = url.toString();
				} catch (MalformedURLException e) {
					mediaurl = "";
					media_file_basename = "";					
				}
			} else {
				mediaurl = file.getParent() + File.separator;
				media_file_basename = file.getName();
			}
		}
	}

	public String getMediaFileName() {
		splitMediaFilename();
		return media_file_basename;
	}

	public String getMediaPrefix() {
		splitMediaFilename();
		return mediaurl;
	}

	public String getSignalFileName() {
		switch(guiMode) {
		case EVALUATION_MODE:
			String ret = jComboBox_signal.getSelectedItem().toString();
			if (ret.isEmpty()) {
				ret = "newfile.nexus";
				jComboBox_signal.setSelectedItem(ret);
			}
			return ret;
		case DIST_MODE:
		case EXPERIMENT_MODE:
			Date date = new Date();
			DateFormat formatter = new SimpleDateFormat("yyyy-MM-dd_HH-mm");
			return formatter.format(date) + ".nexus";
		}
		return null;
	}

	public String getSignalPrefix() {
		return "./recordings/";
	}
	/**
	 * 
	 */
	protected void setComponentFonts(Container c) {
		int num_components = c.getComponentCount();
		for (int i = 0; i < num_components; i++) {
			Component child = c.getComponent(i);
			child.setFont(c.getFont());
			if(child instanceof Container) {
				setComponentFonts((Container)child);
			}
		}
	}

	protected void addEventHandler() {
		eventhandler = new EventHandler(this);
	}
	
	public void saveProperties() {
		if (guiMode == Mode.DIST_MODE) {
			try {
				userProps.setProperty("age", jSpinner_age.getValue().toString());
				userProps.setProperty("trainingtime", jSpinner_time.getValue().toString());
				for (int i = 5; i > 1; i--) {
					userProps.setProperty("lastMedia" + i, userProps.getProperty("lastMedia" + (i - 1), ""));
				}
				userProps.setProperty("lastMedia1", mediaFileTextField.getText());
				userProps.store(new FileWriter(PROPFILE), null);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
}
