package util;

import gui.EventHandler;
import gui.FrontEnd;
import gui.EventHandler.NexusMode;
import gui.EventHandler.PlaybackMode;

import java.awt.Color;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Timer;
import java.util.TimerTask;

import org.cybergarage.upnp.Action;
import org.cybergarage.upnp.Argument;
import org.cybergarage.upnp.ArgumentList;
import org.cybergarage.upnp.Device;
import org.cybergarage.upnp.StateVariable;
import org.cybergarage.upnp.UPnPStatus;
import org.cybergarage.upnp.control.ActionListener;
import org.cybergarage.upnp.control.QueryListener;

public class Runner implements ActionListener, QueryListener {

	private static final String ACTION_START_TRAINING = "StartTraining";
	private static final String ACTION_STOP_TRAINING = "StopTraining";
	private static final String ACTION_IS_RUNNING = "IsRunning";
	private static final String ARG_RUNNING = "Running";
	private static final String ARG_MEDIAFILE = "MediaURL";
	private static final String ARG_SIGNALFILE = "SignalURL";
	private static final String ARG_NEXUSMODE = "NexusMode";
	private static final String ARG_PLAYBACKMODE = "PlaybackMode";
	private static final String ARG_AGE = "Age";
	private static final String ARG_EXTIME = "TrainingTime";
	private static final String VAR_IS_RUNNING = "isRunning";
	
	public static final String MPLAYER_CACHE = Integer.toString(8192);
	

	private static Process videoid = null;
	public static FrontEnd localpanel = null;
	private static boolean isRunning = false;
	
	public static Device upnpDev = null;

	public static void startExercise(String mediaurl, String mediaFileName,
			String signalurl, String signalFileName, NexusMode nexusMode, PlaybackMode playbackMode, String age, int exTime) {
		// The file we want to store the batch-file
		String fileName = "template.bat";
		// And the according id

		// ///////////////////////////////////////
		// Get the media that shall be displayed//
		// ///////////////////////////////////////
		String type = null;

		if (mediaFileName.contains(".")) {
			type = mediaFileName.substring(mediaFileName.lastIndexOf("."));
		} else {
			type = "DVD";
		}

		// ///////////////////////////////////////////////////////////////
		// Get the filename that contains or will contain the biosignals//
		// ///////////////////////////////////////////////////////////////
		if (!signalFileName.endsWith(".nexus")) {
			// System.out.println("Atach .nexus");
			// signalFileName.concat(".nexus");
			signalFileName = signalFileName + ".nexus";
		}
		if (nexusMode == EventHandler.NexusMode.CAPTURE) {
			File testfile = new File(signalurl + signalFileName);
			while (testfile.exists()) {
				System.out.println("File already exists");
				signalFileName = "1" + signalFileName;
				testfile = new File(signalurl + signalFileName);
			}
		}

		// ///////////////////////
		// Get the exercise time//
		// ///////////////////////
		if (exTime == 0) {
			exTime = 2;
		}
		System.out.println("training time= " + exTime);

		// /////////////
		// Get the Age//
		// /////////////
		if (age.isEmpty()) {
			age = "40";
		}

		// The parts that are necessary for the batch file
		String[] part;
		part = new String[9];

		// Do we record
		// Here we need a variable that contains the read/write "*.nexus" file
		switch (nexusMode) {
		case CAPTURE:
			part[0] = "capture_nexus.exe --tee=\"" + signalurl + signalFileName
					+ "\" |";
			break;
		case PLAY:
			part[0] = "capture_nexus.exe --read=\"" + signalurl + signalFileName
					+ "\" |";
			break;
		}

		// The rest is the same for record or play
		// Here we need a selection for the signal (future things)
		part[1] = "nexus_signals.exe --signal=ECG -H 2048 --ignore_disconnect |";
		part[2] = "moving_window.exe --function=ecg_mf_hr --width=0 -H 2048 |";
		part[3] = "fv_select.exe --only-every-nth=2048 -H 1 |";
		part[4] = "fv_select.exe --not-value=0 |";
		part[5] = "moving_window.exe --function=mean --width=10 -H 1 |";
		part[6] = "fv_print.exe -f -H 1 |";
		
		switch (playbackMode) {
		case NOTHING:
			part[7] = " ";
			part[8] = "mplayer.exe -slave -fs -font DejaVuSansMono.ttf Black.wmv";
			break;
		case SIGNAL_ONLY:
			part[7] = "mplayer_interface3.exe " + age + "|";
			part[8] = "mplayer.exe -slave -fs -font DejaVuSansMono.ttf Black.wmv";
			break;
		case SIGNAL_AND_MEDIA:
			part[7] = "mplayer_interface3.exe " + age + "|";
			if (type == "DVD") {
				part[8] = "mplayer.exe -slave -fs -font DejaVuSansMono.ttf  -cache " + MPLAYER_CACHE + " dvd://1 -dvd-device "
						+ mediaurl + mediaFileName;
			} else {
				part[8] = "mplayer.exe -slave -fs -font DejaVuSansMono.ttf -cache " + MPLAYER_CACHE + " \""
						+ mediaurl + mediaFileName + "\"";
			}
			break;
		}

		// ///////////////////////
		// Write the batch file //
		// ///////////////////////
		FileOutputStream writeStream;
		writeStream = null;
		try {
			writeStream = new FileOutputStream(fileName);
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		}

		// Write the batch-file
		for (int i = 0; i < part.length; i++) {
			for (int j = 0; j < part[i].length(); j++) {
				try {
					writeStream.write((byte) part[i].charAt(j));
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		try {
			writeStream.close();
		} catch (IOException e1) {
			e1.printStackTrace();
		}

		// Set the string for the batch-file
		String[] command = { "cmd.exe", "/C", "Start", fileName };

		// //////////////////
		// EXECUTE PROGRAMM//
		// //////////////////
		try {
			// Execute the programm
			System.out.print("Try to load and execxute..     ");

			videoid = Runtime.getRuntime().exec(command);
			isRunning = true;
			updatePanel();
			System.out.println("id: " + videoid);
		} catch (IOException e) {
			e.printStackTrace();
		}

		// ///////////////////////////////////
		// Prepare Stop after Training Time//
		// ///////////////////////////////////
		long delay = exTime * 60 * 1000; // delay for 5 sec.
		// long delay = 10000;
		Timer timer = new Timer();
		// System.out.println("Start:  " + System.currentTimeMillis());
		// System.out.println("delay:  " + delay);
		timer.schedule(new TimerTask() {
			public void run() {
				killStartedApps();
			}
		}, delay);
		System.out.println("...done");
	}

	public static void startExercise(EventHandler.NexusMode nexusMode,
			EventHandler.PlaybackMode playbackMode) {
		startExercise(localpanel.getMediaPrefix(),
				localpanel.getMediaFileName(),
				localpanel.getSignalPrefix(),
				localpanel.getSignalFileName(),
				nexusMode,
				playbackMode,
				localpanel.jSpinner_age.getValue().toString(),
				Integer.parseInt(localpanel.jSpinner_time.getValue().toString())
				);
	}

	public static void killStartedApps() {
		if (videoid == null) {
			isRunning = false;
			return;
		}
		
		String killeradd0 = "process -k capture_nexus.exe";
		String killeradd1 = "process -k fv_print.exe";
		String killeradd2 = "process -k fv_select.exe";
		String killeradd3 = "process -k moving_window.exe";
		String killeradd4 = "process -k mplayer_interface3.exe";
		String killeradd5 = "process -k nexus_signals.exe";
		String playerkiller = "process -k mplayer.exe";
		String commandkiller = "process -k cmd.exe";
		videoid.destroy();
		videoid = null;
		try {
			Runtime.getRuntime().exec(killeradd0);
			Runtime.getRuntime().exec(killeradd1);
			Runtime.getRuntime().exec(killeradd2);
			Runtime.getRuntime().exec(killeradd3);
			Runtime.getRuntime().exec(killeradd4);
			Runtime.getRuntime().exec(killeradd5);
			// System.out.println("End player :  " +
			// System.currentTimeMillis());
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			Runtime.getRuntime().exec(playerkiller);
			Runtime.getRuntime().exec(commandkiller);
			isRunning = false;
			updatePanel();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public boolean actionControlReceived(Action action) {
		String actionName = action.getName();
		ArgumentList argList = action.getArgumentList();
		if (actionName.equals(ACTION_IS_RUNNING)) {
			Argument isRunning = argList.getArgument(ARG_RUNNING);
			isRunning.setValue(Runner.isRunning ? "1" : "0");
			return true;
		}
		if (actionName.equals(ACTION_STOP_TRAINING)) {
			killStartedApps();
			if (upnpDev != null) {
				upnpDev.getStateVariable(VAR_IS_RUNNING).setValue((Runner.isRunning ? "1" : "0"));
			}
			return true;
		}
		if (actionName.equals(ACTION_START_TRAINING)) {
			Argument mediafile = argList.getArgument(ARG_MEDIAFILE);
			Argument signalfile = argList.getArgument(ARG_SIGNALFILE);
			Argument nexusMode = argList.getArgument(ARG_NEXUSMODE);
			Argument playbackMode = argList.getArgument(ARG_PLAYBACKMODE);
			Argument age = argList.getArgument(ARG_AGE);
			Argument exTime = argList.getArgument(ARG_EXTIME);

			NexusMode nexus = NexusMode.PLAY;
			if(nexusMode.getValue().equals("CAPTURE")) {
				nexus = NexusMode.CAPTURE;
			} else if (nexusMode.getValue().equals("PLAY")) {
				nexus = NexusMode.PLAY;
			}
			
			PlaybackMode playback = PlaybackMode.NOTHING;
			if (playbackMode.getValue().equals("NOTHING")) {
				playback = PlaybackMode.NOTHING;
			} else if (playbackMode.getValue().equals("SIGNAL_ONLY")) {
				playback = PlaybackMode.SIGNAL_ONLY;
			} else if (playbackMode.getValue().equals("SIGNAL_AND_MEDIA")) {
				playback = PlaybackMode.SIGNAL_AND_MEDIA;
			}
			
			startExercise("", mediafile.getValue(), "", signalfile.getValue(), nexus, playback, age.getValue(), exTime.getIntegerValue());
			
			if (upnpDev != null) {
				upnpDev.getStateVariable(VAR_IS_RUNNING).setValue((Runner.isRunning ? "1" : "0"));
			}
			return true;
		}
		return false;
	}

	@Override
	public boolean queryControlReceived(StateVariable stateVar) {
		if (stateVar.getName().equals(VAR_IS_RUNNING)) {
			stateVar.setValue((Runner.isRunning ? "1" : "0"));
			return true;
		}
		stateVar.setStatus(UPnPStatus.INVALID_VAR, "No such variable available for querying: <" + stateVar.getName() + ">");
		return false;
	}

	private static void updatePanel() {
		if (localpanel != null) {
			if (isRunning) {
				localpanel.toggle_start.setText("Stop");
				localpanel.toggle_start.setBackground(Color.RED);
			} else {
				localpanel.toggle_start.setText("Start");
				localpanel.toggle_start.setBackground(Color.GREEN);
			}
			localpanel.validate();
		}
	}
}
